//
//  ImageZoomView.m
//  ImageZoom
//
//  Created by zw on 2020/4/13.
//  Copyright © 2020 Zcy. All rights reserved.
//

#import "ImageZoomView.h"
@interface ImageZoomView ()<UIGestureRecognizerDelegate>
@property (nonatomic, strong) UIPanGestureRecognizer *panGesture;//平移手势
@property (nonatomic, strong) UIPanGestureRecognizer *panGesturezoom;//缩放手势
@property (nonatomic, strong) UIImageView *zoombtnimage;
@property (nonatomic, assign) CGPoint lastCtrlPoint;
@property (nonatomic, assign) CGPoint originalPoint;
@property (nonatomic, assign) CGFloat originalWidth;
@end
@implementation ImageZoomView

/*
// Only override drawRect: if you perform custom drawing.
// An empty implementation adversely affects performance during animation.
- (void)drawRect:(CGRect)rect {
    // Drawing code
}
*/
- (instancetype)init{
    if (self = [super init]) {
        [self createSubview];
        [self setupData];
    }
    return self;
}
- (instancetype)initWithFrame:(CGRect)frame{
    if (self = [super initWithFrame:frame]) {
       
        [self createSubview];
        [self setupData];
    }
    return self;
}
- (void)createSubview{
    [self addSubview:self.imageView];
    [self addSubview:self.zoombtnimage];
    [self addViewGesture];
}
- (UIImageView *)imageView{
    if (_imageView == nil) {
        _imageView = [[UIImageView alloc]initWithFrame:CGRectMake(0, 0, self.frame.size.width, self.frame.size.height)];
        _imageView.contentMode = UIViewContentModeScaleToFill;
    }
    return _imageView;
}
- (UIImageView *)zoombtnimage{
    if (_zoombtnimage == nil) {
        _zoombtnimage = [[UIImageView alloc]initWithFrame:CGRectMake(self.frame.size.width-25, self.frame.size.height-25, 30, 30)];
        _zoombtnimage.image = [UIImage imageNamed:@"AddView_rotate"];
        _zoombtnimage.userInteractionEnabled = YES;
        _zoombtnimage.autoresizingMask = UIViewAutoresizingFlexibleLeftMargin | UIViewAutoresizingFlexibleBottomMargin;
        [self bringSubviewToFront:_zoombtnimage];
        UIPanGestureRecognizer *panGesture = [[UIPanGestureRecognizer alloc] initWithTarget:self action:@selector(zoomPanGesture:)];
        [_zoombtnimage addGestureRecognizer:panGesture];
    }
    return _zoombtnimage;
}
- (void)setupData{
  
    //参考点默认为视图中心，point值为中心点比例
   self.originalPoint = CGPointMake(0.5, 0.5);
   self.originalWidth = self.frame.size.height;
//    self.originalHeight = self.frame.size.height;
}
- (void)addViewGesture{
    [self addGestureRecognizer:self.panGesture];
    self.panGesture.delegate = self;
}
//缩放旋转
- (void)zoomPanGesture:(UIPanGestureRecognizer *)panGesture{

    if (panGesture.state == UIGestureRecognizerStateBegan) {
         self.lastCtrlPoint = [self convertPoint:self.zoombtnimage.center toView:self.superview];
        return;
    }
    
    if (panGesture.state == UIGestureRecognizerStateEnded) {
        return;
    }
     CGPoint ctrlPoint = [panGesture locationInView:self.superview];
    CGPoint oPoint = [self convertPoint:[self getRealOriginalPoint] toView:self.superview];
       self.center = oPoint;
       
       //上一个控制点距离中心的距离
       CGFloat preDistance = [self distanceWithStartPoint:self.center endPoint:self.lastCtrlPoint];
       //当前控制点距离中心的距离
       CGFloat newDistance = [self distanceWithStartPoint:self.center endPoint:ctrlPoint];
       CGFloat scale = newDistance / preDistance;
    
    if (scale < 1 && self.frame.size.height <= self.originalWidth) {
           //当缩小到初始化宽高一半时，停止缩小
       }else{
          
           UIView *view=[[UIView alloc]init];
           view.frame=self.frame;
           view.transform=CGAffineTransformScale(self.transform, scale, scale);
           if (view.frame.size.width+self.frame.origin.x>self.superview.frame.size.width) {
               return;
           }
           if (view.frame.size.height+self.frame.origin.y>self.superview.frame.size.height) {
               return;
           }
               self.frame=CGRectMake(self.frame.origin.x, self.frame.origin.y, view.frame.size.width+(view.frame.origin.x-self.frame.origin.x), view.frame.size.height+(view.frame.origin.y-self.frame.origin.y));
            }
           self.imageView.frame=CGRectMake(0, 0, self.frame.size.width, self.frame.size.height);
           self.zoombtnimage.frame =CGRectMake(self.frame.size.width-25, self.frame.size.height-25, 30, 30);
     self.lastCtrlPoint = ctrlPoint;
    [self setNeedsDisplay];
    [panGesture setTranslation:CGPointZero inView:self.superview];
    
}
- (CGPoint)getRealOriginalPoint {
    return CGPointMake(self.bounds.size.width * self.originalPoint.x,
                       self.bounds.size.height * self.originalPoint.y);
}
/* 计算两点间距 */
- (CGFloat)distanceWithStartPoint:(CGPoint)start endPoint:(CGPoint)end {
    CGFloat x = start.x - end.x;
    CGFloat y = start.y - end.y;
    return sqrt(x * x + y * y);
}
- (UIPanGestureRecognizer *)panGesture{
    if (_panGesture == nil) {
        _panGesture = [[UIPanGestureRecognizer alloc]initWithTarget:self action:@selector(panGestureAction:)];
    }
    return _panGesture;
}
//拖动
- (void)panGestureAction:(UIPanGestureRecognizer *)panGesture{
    if (panGesture.state == UIGestureRecognizerStateChanged) {
        CGPoint translation = [panGesture translationInView:self.superview];
        CGPoint newCenter = CGPointMake(panGesture.view.center.x+ translation.x,panGesture.view.center.y + translation.y);//    限制屏幕范围：
        newCenter.y = MAX(panGesture.view.frame.size.height/2, newCenter.y);
        newCenter.y =MIN(self.superview.frame.size.height - panGesture.view.frame.size.height/2,  newCenter.y);
        newCenter.x = MAX(panGesture.view.frame.size.width/2, newCenter.x);
        newCenter.x = MIN(self.superview.frame.size.width - panGesture.view.frame.size.width/2,newCenter.x);
        panGesture.view.center = newCenter;
        [panGesture setTranslation:CGPointZero inView:self.superview];
    }
}
@end
